import argparse


def greet(name):
    print(f"Hello, {name}! Welcome to greet_package!")


def main():
    parser = argparse.ArgumentParser(description="Greet the user by their name.")
    parser.add_argument("name", help="The name of the user to greet.")
    args = parser.parse_args()
    greet(args.name)


if __name__ == "__main__":
    main()



import wx
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from mpl_toolkits.basemap import Basemap
import os
import argparse
from .namelist_wps import mercator_in_wps as miw
from . import namelist as nl


class MyFrame(wx.Frame):
    def __init__(self, parent, title, wps_file):
        super(MyFrame, self).__init__(parent, title=title, size=(1200, 800))
        self.wps_file = wps_file

        # 创建一个Figure对象
        self.figure = Figure(figsize=(2, 2), dpi=100)
        self.canvas = FigureCanvas(self, -1, self.figure)

        # 创建输入框
        self.e_we_box = wx.TextCtrl(self, value="150", size=(100, 30))
        self.e_sn_box = wx.TextCtrl(self, value="150", size=(100, 30))
        self.ref_lat_box = wx.TextCtrl(self, value="23", size=(100, 30))
        self.ref_lon_box = wx.TextCtrl(self, value="112", size=(100, 30))
        self.true_lat_box = wx.TextCtrl(self, value="35", size=(100, 30))
        self.stand_lon_box = wx.TextCtrl(self, value="110", size=(100, 30))
        self.dx_box = wx.TextCtrl(self, value="20000", size=(100, 30))

        # 创建标签
        self.e_we_label = wx.StaticText(self, label="e_we:")
        self.e_sn_label = wx.StaticText(self, label="e_sn:")
        self.ref_lat_label = wx.StaticText(self, label="ref_lat:")
        self.ref_lon_label = wx.StaticText(self, label="ref_lon:")
        self.true_lat_label = wx.StaticText(self, label="true_lat:")
        self.stand_lon_label = wx.StaticText(self, label="stand_lon:")
        self.dx_label = wx.StaticText(self, label="dx:")

        # 创建按钮
        self.update_button = wx.Button(
            self, label="Update Map", size=(150, 30))
        self.update_button.Bind(wx.EVT_BUTTON, self.on_update_button_click)

        # 用网格布局
        grid_sizer = wx.GridBagSizer(vgap=10, hgap=10)
        grid_sizer.Add(self.e_we_label, pos=(0, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.e_we_box, pos=(0, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.e_sn_label, pos=(1, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.e_sn_box, pos=(1, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.ref_lat_label, pos=(2, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.ref_lat_box, pos=(2, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.ref_lon_label, pos=(3, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.ref_lon_box, pos=(3, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.dx_label, pos=(4, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.dx_box, pos=(4, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.true_lat_label, pos=(5, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.true_lat_box, pos=(5, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.stand_lon_label, pos=(6, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.stand_lon_box, pos=(6, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        grid_sizer.Add(self.update_button, pos=(7, 0), span=(1, 2), flag=wx.ALIGN_CENTER|wx.ALL, border=5)
    
        # 创建提示信息
        info = '''投影方式只支持Mercator；\nstand_lon最好和ref_lon比较接近；\n在Mercator投影中，dy无论\n设置多少，都是等于dx的'''
        rich_text = wx.StaticText(self, label=info, style=wx.ALIGN_LEFT|wx.LEFT)

        # 创建显示边界的输入框
        north_label = wx.StaticText(self, label='北')
        south_label = wx.StaticText(self, label='南')
        west_label = wx.StaticText(self, label='西')
        east_label = wx.StaticText(self, label='东')
        bound_info_label = wx.StaticText(self, label='计算得到的边界（不能修改）')
        self.north_box = wx.TextCtrl(self, value="40", size=(100, 30), style=wx.TE_READONLY)
        self.south_box = wx.TextCtrl(self, value="20", size=(100, 30), style=wx.TE_READONLY)
        self.west_box = wx.TextCtrl(self, value="100", size=(100, 30), style=wx.TE_READONLY)
        self.east_box = wx.TextCtrl(self, value="120", size=(100, 30), style=wx.TE_READONLY)
        bound_grid_sizer = wx.GridBagSizer(vgap=10, hgap=10)
        bound_grid_sizer.Add(bound_info_label, pos=(0, 0), span=(1, 2), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(north_label, pos=(1, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(self.north_box, pos=(1, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(south_label, pos=(2, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(self.south_box, pos=(2, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(west_label, pos=(3, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(self.west_box, pos=(3, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(east_label, pos=(4, 0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)
        bound_grid_sizer.Add(self.east_box, pos=(4, 1), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALL, border=5)

        # 创建保存文件与读取文件
        self.path_str = wx.StaticText(self, label='未选择文件', size=(200, 50))
        self.hint_str = wx.StaticText(self, label='', size=(200, 50))
        open_file_button = wx.Button(self, label="Open WPS File")
        open_file_button.Bind(wx.EVT_BUTTON, self.on_open_file)
        save_file_button = wx.Button(self, label="Save File")

        # 创建BoxSizer
        horizontal_sizer = wx.BoxSizer(wx.HORIZONTAL)
        vertical_sizer1 = wx.BoxSizer(wx.VERTICAL)
        vertical_sizer2 = wx.BoxSizer(wx.VERTICAL)
        # 设置布局
        vertical_sizer1.Add(rich_text, proportion=0)
        vertical_sizer1.Add(wx.StaticLine(self, style=wx.HORIZONTAL), 0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        vertical_sizer1.Add(bound_grid_sizer, proportion=0, flag=wx.EXPAND)
        vertical_sizer2.Add(self.path_str, proportion=0, flag=wx.EXPAND)
        vertical_sizer2.Add(self.hint_str, proportion=0, flag=wx.EXPAND)
        vertical_sizer2.Add(open_file_button, 0, wx.EXPAND | wx.BOTTOM, 20)
        vertical_sizer2.Add(save_file_button, proportion=0, flag=wx.EXPAND)
        vertical_sizer2.Add(wx.StaticLine(self, style=wx.HORIZONTAL), 0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        vertical_sizer2.Add(grid_sizer, proportion=1, flag=wx.EXPAND) 
        horizontal_sizer.Add(self.canvas, proportion=1, flag=wx.EXPAND)
        horizontal_sizer.Add(vertical_sizer1, proportion=0,flag=wx.ALL | wx.EXPAND, border=20)
        horizontal_sizer.Add(wx.StaticLine(self, style=wx.VERTICAL), 0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        horizontal_sizer.Add(vertical_sizer2, proportion=0,flag=wx.ALL | wx.EXPAND, border=20)

        # 设置窗口的主 sizer
        self.SetSizer(horizontal_sizer)

        # 使用Basemap创建地图
        self.ax = self.figure.add_subplot(111)
        self.draw_map()

    def on_update_button_click(self, event):
        self.ax.clear()
        self.draw_map()

    def on_open_file(self, event):
        # 定义文件路径
        if self.wps_file:
            wps_file = os.path.abspath(self.wps_file)
            folder_path = os.path.dirname(wps_file)
            file_name = os.path.basename(wps_file)
        else:
            folder_path = os.getcwd()
            file_name = 'namelist.wps'
        # 创建文件选择器对话框
        file_dialog = wx.FileDialog(self, message="Choose a file",
                                    defaultDir=folder_path,
                                    defaultFile=file_name,
                                    wildcard="*.*",
                                    style=wx.FD_OPEN)

        # 显示文件选择器，如果用户点击“确定”，则获取选择的文件路径
        if file_dialog.ShowModal() == wx.ID_OK:
            selected_file = file_dialog.GetPath()
            print(f"Selected file: {selected_file}")
            self.load_wps_file(selected_file)
            self.path_str.SetLabel(selected_file)
            self.Layout()
            self.nl_file = selected_file

        # 销毁文件选择器以释放资源
        file_dialog.Destroy()

    def on_save_file(self, event):
        nl.modify(self.nl_file, 'e_we', self.e_we_box.GetValue())
        nl.modify(self.nl_file, 'e_sn', self.e_sn_box.GetValue())
        nl.modify(self.nl_file, 'ref_lat', self.ref_lat_box.GetValue())
        nl.modify(self.nl_file, 'ref_lon', self.ref_lon_box.GetValue())
        nl.modify(self.nl_file, 'truelat1', self.true_lat_box.GetValue())
        nl.modify(self.nl_file, 'stand_lon', self.stand_lon_box.GetValue())
        nl.modify(self.nl_file, 'dx', self.dx_box.GetValue())
        nl.modify(self.nl_file, 'dy', self.dx_box.GetValue())
        self.hint_str.SetLabel('修改成功')

    def load_wps_file(self, file_path):
        try:
            config = nl.load(file_path)
            # 读取namelist.wps中的参数
            ref_lat = config['geogrid']['ref_lat']
            ref_lon = config['geogrid']['ref_lon']
            true_lat = config['geogrid']['truelat1']
            stand_lon = config['geogrid']['stand_lon']
            delta = config['geogrid']['dx']
            e_we = config['geogrid']['e_we']
            e_sn = config['geogrid']['e_sn']
            self.e_we_box.SetValue(str(e_we))
            self.e_sn_box.SetValue(str(e_sn))
            self.ref_lat_box.SetValue(str(ref_lat))
            self.ref_lon_box.SetValue(str(ref_lon))
            self.true_lat_box.SetValue(str(true_lat))
            self.stand_lon_box.SetValue(str(stand_lon))
            self.dx_box.SetValue(str(delta))
            self.hint_str.SetLabel('namelist.wps文件成功加载')
            self.on_update_button_click(None)
        except KeyError:
            self.hint_str.SetLabel('namelist.wps文件读取失败')

    def draw_map(self):
        # 获取输入框的值
        e_we = int(self.e_we_box.GetValue())
        e_sn = int(self.e_sn_box.GetValue())
        ref_lat = float(self.ref_lat_box.GetValue())
        ref_lon = float(self.ref_lon_box.GetValue())
        dx = float(self.dx_box.GetValue())
        true_lat = float(self.true_lat_box.GetValue())
        north, west, south, east = miw(
            ref_lat, ref_lon, true_lat, dx, e_we, e_sn)

        self.map = Basemap(
            projection='merc',
            llcrnrlat=south,
            urcrnrlat=north,
            llcrnrlon=west,
            urcrnrlon=east,
            lat_ts=true_lat,
            resolution='l',
            ax=self.ax
        )
        self.map.drawcoastlines()
        self.map.drawcountries()
        self.map.fillcontinents(color='lightgray', lake_color='aqua', zorder=1)
        self.map.drawmapboundary(fill_color='white')

        # 绘制经纬线
        self.map.drawparallels(range(-90, 90, 5), labels=[
                               1, 0, 0, 0], linewidth=0.5, color='gray')
        self.map.drawmeridians(range(-180, 180, 5), labels=[
                               0, 0, 0, 1], linewidth=0.5, color='gray')

        # 更新画布
        self.canvas.draw()

        # 更新边界输入框的值
        self.north_box.SetValue(str(north))
        self.south_box.SetValue(str(south))
        self.west_box.SetValue(str(west))
        self.east_box.SetValue(str(east))


def main():
    parser = argparse.ArgumentParser(description='这是一个计算namelist.wps经纬度范围的程序')
    parser.add_argument('file', nargs='?', default=None, type=str, help='输入的namelist.wps文件')
    args = parser.parse_args()
    app = wx.App()
    frame = MyFrame(None, '地图投影计算器', args.file)
    frame.Show()
    app.MainLoop()


if __name__ == '__main__':
    main()



import ftplib
import time
import threading
import argparse
import os.path as op
import os
from concurrent.futures import ThreadPoolExecutor


class LinkFTP:
    def __init__(self, name='ftpA'):
        self.f = ftplib.FTP()
        self.f.connect('ftp.avl.class.noaa.gov')
        self.f.login('', '')
        self.downloading = 0
        self.name = name
        self.lock = threading.Lock()
        self.num = 1
        print(f'{self.name}初始化完成')

    def download_file(self, remote_path, local_path):
        # 这里必须进行加锁处理，同一个FTP对象只能同时获取一个文件
        self.lock.acquire()
        print(self.name, '将下载其自己的第', self.num, '个文件', remote_path)
        with open(local_path, 'wb') as fp:
            self.f.retrbinary('RETR '+remote_path, fp.write, 2048)
        self.num += 1
        self.lock.release()

    def get_filenames(self, dir='./'):
        filenames = [filename for filename,
                     info in self.f.mlsd(dir) if info['type'] == 'file']
        return filenames

    def quit(self):
        self.f.close()


class FTPPooling:
    def __init__(self, num_ftp):
        self.num_ftp = num_ftp
        with ThreadPoolExecutor(10) as t:
            res = t.map(lambda i: LinkFTP(f'ftp{i}'), list(range(num_ftp)))
            self.ftps = list(res)
        self.threads = []

    def close(self):
        for f in self.ftps:
            f.quit()

    def setFiles(self, filepath, filenames):
        files_full_path = [f"{filepath}/{filename}" for filename in filenames]
        self.filenames = filenames
        self.filenames_group = [files_full_path[i:i+self.num_ftp]
                                for i in range(0, len(files_full_path), self.num_ftp)]

    def set_save_path(self, local_path='./', sub_dir=None):
        self.save_path = local_path
        if sub_dir:
            self.save_path = op.join(local_path, sub_dir)
        if not op.exists(self.save_path):
            os.makedirs(self.save_path)
            print('目标目录不存在，将创建目录')

    def download(self):
        for group in self.filenames_group:
            for i, filename in enumerate(group):
                basename = op.basename(filename)
                t = threading.Thread(target=self.ftps[i].download_file, args=(
                    filename, op.join(self.save_path, basename)))
                t.start()
                self.threads.append(t)
                time.sleep(0.01)
        self.wait()

    def wait(self):
        for t in self.threads:
            t.join()

    def quit(self):
        for f in self.ftps:
            f.quit()


def main(order_id, num_ftp, save_path):
    '''
    order_id: 订单号
    num_ftp: ftp最大数量
    save_path: 保存路径
    '''
    # 测试FTP可用性，以及获取对应订单的文件
    print(f'订单号为：{order_id}')
    print('-------------------')
    fi = LinkFTP('ftp测试用例')
    filepath = f'/{order_id}/001'  # 对面FTP服务上的文件夹路径
    filenames = fi.get_filenames(filepath)
    print(f'订单{order_id}有{len(filenames)}个文件')
    fi.quit()
    # 下载文件
    num_ftp = min(num_ftp, len(filenames))
    print(f'将使用{num_ftp}个ftp下载')
    print(f'保存路径为{save_path}')
    print('-------------------')
    # 下载文件
    ftp_pool = FTPPooling(num_ftp)
    ftp_pool.setFiles(filepath, filenames)
    ftp_pool.set_save_path(save_path)
    ftp_pool.download()
    ftp_pool.close()

    print("全部下载完成")


def commond():
    '''
    命令行调用
    '''
    parser = argparse.ArgumentParser(description='下载NOAA数据，比如ATMS L1 L2的数据')
    parser.add_argument('order', type=str, help='订单编号')
    parser.add_argument('--nftp', '-n', default=30,
                        type=int, help='ftp最大数量，默认为%(default)s')
    parser.add_argument('-o', dest='save_path', metavar='PATH',
                        default='./', type=str, help='保存路径，默认为当前路径')
    args = parser.parse_args()

    order_id = args.order
    num_ftp = args.nftp
    save_path = args.save_path
    main(order_id, num_ftp, save_path)


if __name__ == '__main__':
    commond()



"""
mylogger.py: 一个自定义日志记录器模块，提供了两种日志记录器：标准日志记录器和彩色日志记录器。

此模块包含以下功能：

1. 标准日志记录器：输出具有自定义格式的日志信息。
2. 彩色日志记录器：根据日志级别输出不同颜色的日志信息。
3. get_logger() 函数：提供日志记录器名称，返回相应的日志记录器实例。

使用示例：

导入模块并获取所需的日志记录器，然后使用该记录器记录日志信息。

from mylogger import get_logger

logger = get_logger('colorful')
logger.info("这是一条信息级别的日志")
logger.warning("这是一条警告级别的日志")
logger.error("这是一条错误级别的日志")
"""
import logging
from colorlog import ColoredFormatter
import sys
import os
import time


# 确保日志打印的时间为北京时间
os.environ["TZ"] = "Asia/Shanghai"
time.tzset()


def remove_logger_by_name(logger_name):
    '''
    删除已经存在的logger
    :param logger_name: logger名称
    :return: 是否删除成功
    '''
    if logger_name in logging.Logger.manager.loggerDict:
        del logging.Logger.manager.loggerDict[logger_name]
        return True
    return False


def standard_logger():
    # 自定义日志记录器类
    class CustomFormatter(logging.Formatter):
        def format(self, record):
            levelname = record.levelname
            record.levelname = f"[{levelname}]".ljust(9)
            return super().format(record)

    # 自定义格式化字符串
    log_format = "%(asctime)s %(levelname)s %(message)s"
    date_format = "%Y-%m-%d %H:%M:%S"

    # 设置日志级别
    remove_logger_by_name('standard')
    logger = logging.getLogger('standard')
    logger.setLevel(logging.DEBUG)

    # 设置日志处理器和格式化器
    handler = logging.StreamHandler(sys.stdout)
    handler.setFormatter(CustomFormatter(log_format, datefmt=date_format))
    logger.addHandler(handler)
    return logger


def colorful_logger():
    LOG_FORMAT = "%(asctime)s %(log_color)s[%(levelname)s] %(message)s"
    formatter = ColoredFormatter(
        LOG_FORMAT,
        datefmt="%Y-%m-%d %H:%M:%S",
        log_colors={
            'DEBUG': 'cyan',
            'INFO': 'green',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'red,bg_white',
        }
    )

    # 配置彩色logger
    remove_logger_by_name('colorful')
    logger = logging.getLogger('colorful')
    logger.setLevel(logging.DEBUG)

    # Set up the console handler
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    return logger


def get_logger(name):
    if name == 'standard':
        return standard_logger()
    elif name == 'colorful':
        return colorful_logger()
    else:
        raise ValueError('不合法的logger name: %s' % name)



import numpy as np
from pyproj import CRS, Transformer
import re


def get_zenith_angle(sat_lon, sat_lat, sat_alt, obs_lon, obs_lat, obs_alt=None):
    """
    计算卫星与地面观测点之间的天顶角（本地入射角）
    :参数 sat_lon: 卫星经度（单位：度）
    :参数 sat_lat: 卫星纬度（单位：度）
    :参数 sat_alt: 卫星高度（单位：千米）
    :参数 obs_lon: 观测点经度（单位：度）
    :参数 obs_lat: 观测点纬度（单位：度）
    :参数 obs_alt: 观测点高度（单位：米），可选，默认为0
    :return: 卫星与地面观测点之间的天顶角（单位：度）
    """

    # 预处理
    obs_shape = obs_lat.shape
    obs_lat = obs_lat.flatten()
    obs_lon = obs_lon.flatten()
    obs_alt = obs_alt.flatten() if obs_alt is not None else np.zeros_like(obs_lat)
    sat_alt = sat_alt * 1000  # 卫星高度:km转换为米

    # 创建坐标转换器（从WGS84经纬度坐标转换到地心惯性坐标系）
    crs_lla = CRS("EPSG:4326")  # WGS84经纬度坐标系统
    crs_ecef = CRS("EPSG:4978")  # 地心惯性坐标系
    transformer = Transformer.from_crs(crs_lla, crs_ecef, always_xy=True)

    # 将卫星位置、地面观测点位置从WGS84经纬度和高度转换为地心惯性坐标系下的坐标
    sat_ecef = transformer.transform(sat_lon, sat_lat, sat_alt)
    obs_ecef = transformer.transform(obs_lon, obs_lat, obs_alt)

    # 计算卫星与地面观测点之间的向量
    obs_vec = np.array(obs_ecef)
    obs_sat_vec = np.array(sat_ecef)[:, np.newaxis] - obs_vec
    obs_sat_dist = np.linalg.norm(obs_sat_vec, axis=0)
    obs_dist = np.linalg.norm(obs_vec, axis=0)

    # 计算卫星与地面观测点之间的夹角（单位：度）
    theta = np.rad2deg(
        np.arccos(np.sum(obs_vec*obs_sat_vec, axis=0) / obs_sat_dist / obs_dist))

    return theta.reshape(obs_shape)


class ShellScriptVariable:
    '''
    这个类用于读取和修改shell脚本中的变量值。可以通过实例化对象或使用类方法来完成操作。
    '''
    def __init__(self, script_path, **kwargs):
        '''
        初始化方法，为对象赋予脚本路径，并读取脚本内容。
        :param script_path: shell脚本的路径
        :param **kwargs: 传递给打开文件的其他参数
        '''
        self.script_path = script_path
        with open(script_path, 'r', **kwargs) as f:
            self.content = f.read()

    def get(self, variable_name):
        '''
        获取脚本中指定变量的值。
        :param variable_name: 需要获取的变量名
        :return: 返回变量值
        :raises ValueError: 如果变量不存在，抛出ValueError异常
        '''
        pattern = rf"({variable_name}\s*=\s*)([^\s\n][^\n]*)"
        match = re.search(pattern, self.content)
        if match:
            return match.group(2)
        else:
            raise ValueError(f"变量 {variable_name} 不存在")

    def update(self, variable_name, new_value, count=1):
        '''
        更新脚本中指定变量的值。
        :param variable_name: 需要更新的变量名
        :param new_value: 变量更新后的值
        :param count: 需要更新的匹配项数量，默认为1，设置为0时将更新所有匹配项
        '''
        pattern = rf"({variable_name}\s*=\s*)([^\s\n][^\n]*)"
        # 确认是否存在变量，不存在会抛出异常
        self.get(variable_name)
        new_str = f"{variable_name}={new_value}"
        self.content = re.sub(pattern, new_str, self.content, count=count)

    def save(self, save_path=None):
        '''
        将修改后的脚本内容保存到文件。
        :param save_path: 保存的路径，默认为None，将覆盖原始脚本文件
        '''

        if not save_path:
            save_path = self.script_path
        with open(save_path, 'w') as f:
            f.write(self.content)

    def show(self):
        '''
        显示脚本的内容。
        '''
        print(f'-------{self.script_path}----------')
        print(self.content)
        print('------------------------------------')

    @classmethod
    def modify(cls, script_path, variable_name, new_value, save_path=None, count=1, **kwargs):
        '''
        类方法，修改脚本的变量值并保存。
        :param script_path: shell脚本的路径
        :param variable_name: 需要更新的变量名
        :param new_value: 变量更新后的值
        :param save_path: 保存的路径，默认为None，将覆盖原始脚本文件
        :param count: 需要更新的匹配项数量，默认为1，设置为0时将更新所有匹配项
        :param **kwargs: 传递给打开文件的其他参数
        '''
        ssv = cls(script_path, **kwargs)
        ssv.update(variable_name, new_value, count=count)
        ssv.save(save_path)



from pyproj import Transformer
from . import namelist as nl
from math import ceil, floor
import argparse
import os


def mercator_in_wps(rlat, rlon, tlat, delta, e_we, e_sn):
    '''
    用于计算namelist.wps中的参数计算经纬度范围
    rlat: 中心纬度
    rlon: 中心纬度
    tlat: 真实经纬度true_lat
    delta: 网格间距dx,dy你设置成不一样的，也会是一样的
    e_we: 东西方向格点数
    e_sn: 南北方向格点数
    返回：north, west, south, east
    '''
    # 创建自定义麦卡托投影的定义字符串
    mercator_proj = f"+proj=merc +lon_0=100 +lat_ts={tlat} +x_0=0 +y_0=0 +ellps=WGS84 +units=m +no_defs"

    # 创建转换器实例
    tf1 = Transformer.from_crs("EPSG:4326", mercator_proj)
    tf2 = Transformer.from_crs(mercator_proj, "EPSG:4326")

    # 将经纬度坐标转换为自定义麦卡托投影坐标
    rx, ry = tf1.transform(rlat, rlon)

    north, west = tf2.transform(rx - delta*e_we / 2, ry + delta*e_sn / 2)
    south, east = tf2.transform(rx + delta*e_we / 2, ry - delta*e_sn / 2)

    return north, west, south, east


def main():
    parser = argparse.ArgumentParser(description='这是一个计算namelist.wps经纬度范围的程序')
    parser.add_argument('file', nargs='?', type=str, help='输入的namelist.wps文件')
    args = parser.parse_args()

    wps_file = args.file
    if wps_file is None:
        wps_file = os.path.join(os.getcwd(), 'namelist.wps')
        print(f"未设置文件路径，将尝试在当前路径下读取{wps_file}")
    try:
        config = nl.load(wps_file)
    except FileNotFoundError:
        print(f"文件{wps_file}不存在，请检查文件路径是否正确")
        exit(1)

    # 读取namelist.wps中的参数
    map_proj = config['geogrid']['map_proj']
    ref_lat = config['geogrid']['ref_lat']
    ref_lon = config['geogrid']['ref_lon']
    true_lat = config['geogrid']['truelat1']
    stand_lon = config['geogrid']['stand_lon']
    delta = config['geogrid']['dx']
    e_we = config['geogrid']['e_we']
    e_sn = config['geogrid']['e_sn']
    # 诊断信息
    if map_proj == 'mercator':
        print("error 本命令支持麦克托投影方式")
    # 计算经纬度范围
    north, west, south, east = mercator_in_wps(ref_lat, ref_lon, true_lat, delta, e_we, e_sn)
    # 诊断信息
    oppsite_lon = (stand_lon + 180) - 360 * (stand_lon + 180 > 180)
    if oppsite_lon < east and oppsite_lon > west:
        print("stand_lon应该有问题，应该设置比较接近中心经度的值，你可以使用plotgrids.ncl来确认经纬度范围")
    # 向外取整
    north = ceil(north)
    west = floor(west)
    south = floor(south)
    east = ceil(east)
    print(f"向外取整后： 北{north},  西{west},  南{south},  东{east}")


if __name__ == "__main__":
    main()
